package fr.natan.microservices.springsecurityserverjwt.t_authenticationFilter.b_authenticationFilter;

import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.Instant;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

import static org.springframework.util.MimeTypeUtils.APPLICATION_JSON_VALUE;


public class JWTAuthenticationFilter extends UsernamePasswordAuthenticationFilter {
    private final AuthenticationManager authenticationManager;

    public JWTAuthenticationFilter(AuthenticationManager authenticationManager) {
        this.authenticationManager = authenticationManager;
    }

    //la methode attemptAuthentication crée un object de type Authentication contenant login et password
    @Override
    public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response)
            throws AuthenticationException {
        String username = request.getParameter(FinalVariables.USERNAME);
        String password = request.getParameter(FinalVariables.PWD);
        UsernamePasswordAuthenticationToken authenticationToken = new
                UsernamePasswordAuthenticationToken(username, password);

        return authenticationManager.authenticate(authenticationToken);
    }


    //si l'authentification a réussi, Spring Chain Filter récupère le user authentifié
    // et construit le JWT
    @Override
    protected void successfulAuthentication(
            HttpServletRequest request, HttpServletResponse response,
            FilterChain chain, Authentication authResult
    ) throws IOException {

        User user = (User) authResult.getPrincipal();//récupération du user authentifié
        //on commence ici la création du JWT
        Algorithm algorithm = Algorithm.HMAC256(FinalVariables.SECRET.getBytes());
        Instant now = Instant.now();
        String JWTAccessToken = JWT.create()
                .withSubject(user.getUsername())
                .withIssuedAt(Date.from(now))
                .withExpiresAt(now.plusMillis(FinalVariables.ACCESS_TOKEN_EXPIRED_AT))
                .withIssuer(request.getRequestURL().toString())
                .withClaim(FinalVariables.ROLES, user.getAuthorities().stream()
                        .map(grantedAuthority -> grantedAuthority.getAuthority())
                        .collect(Collectors.toList()))
                .sign(algorithm);


        String JWTRefreshToken = JWT.create()
                .withSubject(user.getUsername())
                .withIssuedAt(Date.from(now))
                .withExpiresAt(Date.from(now.plusMillis(FinalVariables.REFRESH_TOKEN_EXPIRED_AT)))
                .withIssuer(request.getRequestURL().toString())
                .sign(algorithm);
        Map<String, String> tokens = new HashMap<>(){
            {
                put(FinalVariables.ACCESS_TOKEN, JWTAccessToken);
                put(FinalVariables.REFRESH_TOKEN, JWTRefreshToken);
            }
        };
        response.setContentType(APPLICATION_JSON_VALUE);
        new ObjectMapper().writeValue(response.getOutputStream(),tokens);
    }
}

