package fr.natan.microservices.springsecurityserverjwt.t_authenticationFilter.c_filterConfig;

import fr.natan.microservices.springsecurityserverjwt.service.services.AccountService;
import fr.natan.microservices.springsecurityserverjwt.t_authenticationFilter.a_userCreation.MyUserDetailsServiceImpl;
import fr.natan.microservices.springsecurityserverjwt.t_authenticationFilter.b_authenticationFilter.FinalVariables;
import fr.natan.microservices.springsecurityserverjwt.t_authenticationFilter.b_authenticationFilter.JWTAuthenticationFilter;
import fr.natan.microservices.springsecurityserverjwt.tt_authorizationFilter.JWTAuthorizationFilter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.configuration.AuthenticationConfiguration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import static org.springframework.http.HttpMethod.GET;
import static org.springframework.http.HttpMethod.POST;

@Configuration
@EnableWebSecurity
public class SpringFilterChain {

    //configuration des droits d'accès aux endpoints des request avec HttpSecurity
    //Dans cette méthode securityFilterChain(), on y met toutes les filtres définis
    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity httpSecurity) throws Exception {

        SecurityFilterChain filteredSecurityChain =
                httpSecurity
                        .csrf(csrf -> csrf.disable())
                        .headers(header -> header.frameOptions().disable())
                        .sessionManagement(session -> session
                                .sessionCreationPolicy(SessionCreationPolicy.STATELESS))
                        .authorizeHttpRequests(httpRequest ->
                                httpRequest
                                        .antMatchers(GET, FinalVariables.ENDPOINTS[0]).hasAnyAuthority(
                                                FinalVariables.AUTHORITIES
                                        )
                                        .antMatchers(POST, FinalVariables.ENDPOINTS[0]).hasAnyAuthority(
                                                FinalVariables.AUTHORITIES[1],
                                                FinalVariables.AUTHORITIES[2],
                                                FinalVariables.AUTHORITIES[3])
                                        .anyRequest().authenticated()
                        )
                        //ajout du filtre d'authentification
                        .addFilter(new JWTAuthenticationFilter(authenticationManager()))
                        .addFilterBefore(new JWTAuthorizationFilter(), UsernamePasswordAuthenticationFilter.class)
                        .build();
        return filteredSecurityChain;
    }

    //appel du UserDetails service pour construire un user Spring
    @Bean
    public MyUserDetailsServiceImpl configureUser(AccountService accountService) {
        return new MyUserDetailsServiceImpl(accountService);
    }

    private final AuthenticationConfiguration authenticationConfiguration;

    public SpringFilterChain(AuthenticationConfiguration authenticationConfiguration) {
        this.authenticationConfiguration = authenticationConfiguration;
    }

    public AuthenticationManager authenticationManager() throws Exception {
        return authenticationConfiguration.getAuthenticationManager();
    }
}
