package fr.natan.microservices.springsecurityserverjwt.tt_authorizationFilter;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.fasterxml.jackson.databind.ObjectMapper;
import fr.natan.microservices.springsecurityserverjwt.t_authenticationFilter.b_authenticationFilter.FinalVariables;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;

import static org.springframework.http.HttpStatus.FORBIDDEN;
import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;


public class JWTAuthorizationFilter extends OncePerRequestFilter {
    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {
        //get JWT send by client
        String jwtAccessToken = request.getHeader(FinalVariables.AUTHORIZATION);
        if(jwtAccessToken!=null && jwtAccessToken.startsWith(FinalVariables.START_WITH)){
           try {
               String jwtToken = jwtAccessToken.substring(FinalVariables.START_WITH.length());
               Algorithm algorithm = Algorithm.HMAC256(FinalVariables.SECRET.getBytes());
               JWTVerifier jwtVerifier = JWT.require(algorithm).build();
               DecodedJWT decodedJWT = jwtVerifier.verify(jwtToken);
               //if decodedJWT is valid, that is, not expired, ...do else catch
               String username = decodedJWT.getSubject();
               String [] roles = decodedJWT.getClaim(FinalVariables.ROLES).asArray(String.class);
               List<GrantedAuthority> grantedAuthorities = new ArrayList<>();
              for(String role: roles){
                  grantedAuthorities.add(new SimpleGrantedAuthority(role));
              }
               //pour vérifier l'utilisateur, le pwd n'est pas nécessaire, ici null,
               //on a juste besoin du username et ses roles
               UsernamePasswordAuthenticationToken usernamePasswordAuthenticationToken =
                       new UsernamePasswordAuthenticationToken(username, null, grantedAuthorities);
               SecurityContextHolder.getContext().setAuthentication(usernamePasswordAuthenticationToken);
               filterChain.doFilter(request, response);


           }catch (Exception exception){
               response.setHeader("error-message", exception.getMessage());
               Map<String, String> errors = new HashMap<>();
               errors.put("error-message", exception.getMessage());
               response.setContentType(APPLICATION_JSON_VALUE);
               new ObjectMapper().writeValue(response.getOutputStream(), errors);
               response.sendError(FORBIDDEN.value());
           }
        }
        else{
            //go to the next request if exists
            filterChain.doFilter(request,response);
        }
    }
}
