package fr.natan.microservices.springsecurityserverjwt.web;

import fr.natan.microservices.springsecurityserverjwt.models.AppRole;
import fr.natan.microservices.springsecurityserverjwt.models.UserAccount;
import fr.natan.microservices.springsecurityserverjwt.models.UserAccountRoleForm;
import fr.natan.microservices.springsecurityserverjwt.moduleExceptions.businessExceptions.*;
import fr.natan.microservices.springsecurityserverjwt.service.services.AccountService;
import org.springframework.security.access.prepost.PostAuthorize;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.security.Principal;
import java.util.Collection;

@RestController
public class AccountController {
    private final AccountService accountService;

    public AccountController(AccountService accountService) {
        this.accountService = accountService;
    }

    @RequestMapping(value = "/users", method = RequestMethod.POST)
    private UserAccount createUserAccount(@RequestBody UserAccount userAccount)
            throws UserAccountAlreadyExistsException, UserAccountFieldsInvalidException {
        return accountService.addNewUserAccount(userAccount);
    }

    @RequestMapping(value = "/roles", method = RequestMethod.POST)
    private AppRole createRole(@RequestBody AppRole appRole) throws RoleAlreadyExistsException, RoleFieldsInvalidException {
        return accountService.addNewRole(appRole);
    }

    @RequestMapping(value = "/add-role-to-user", method = RequestMethod.POST)
    private void addRoleToUser(@RequestBody UserAccountRoleForm userAccountRoleForm) throws AppRoleNotFoundException,
            UserAccountNotFoundException, UserAccountAlreadyAssignedThatRoleException {
        accountService.addRoleToUser(userAccountRoleForm.getUsername(), userAccountRoleForm.getRoleName());
    }

    @RequestMapping(value = "/users", method = RequestMethod.GET)
    private Collection<UserAccount> listUserAccounts(){
        return accountService.loadAllUsers();
    }

    @RequestMapping(value = "/roles", method = RequestMethod.GET)
    private Collection<AppRole> listRoles(){
        return accountService.loadAllRoles();
    }

    @RequestMapping(value = "/profil", method = RequestMethod.GET)
    private UserAccount getProfil(Principal principal) throws UserAccountNotFoundException {
        return accountService.getUserByUsername(principal.getName());
    }

    //develop here refresh token
}
